@@ -1,7 +1,60 @@
 from time import time
+import json
 
 
+def strToCqCode(message):
+    """
+    提取字符串中的 cqCode 字符串
+    """
+    msg = list(message)
+    cq_code = ""
+    cq_code_in = False
+    cq_code_list = []
+    for str in msg:
+        if cq_code_in and str != "]":
+            cq_code += str
+            continue
+        
+        if cq_code_in and str == "]":
+            cq_code += str
+            cq_code_list.append(cq_code)
+            cq_code = ""
+            cq_code_in = False
+            continue
+
+        if str == "[":
+            cq_code = ""
+            cq_code += str
+            continue
+
+        if str == "C":
+            cq_code += str
+            continue
+
+        if str == "Q":
+            cq_code += str
+            continue
+
+        if str == ":" and cq_code == "[CQ":
+            cq_code += str
+            cq_code_in = True
+
+    return cq_code_list
+
+def strToCqCodeToDict(message):
+    """
+    提取字符串中的 cqCode 字符串转换为字典
+    """
+    CqCodeList = strToCqCode(message)
+    for count in range(0, len(CqCodeList)):
+        CqCodeList[count] = get_cq_code(CqCodeList[count])
+    
+    return CqCodeList
+
 def set_cq_code(code):
+    """
+    转换 pycqBot 的 cqCode 字典为 cqCode 字符串
+    """
     data_str = ""
     for key in code["data"].keys():
         data_str += ",%s=%s" % (key, code["data"][key])
@@ -9,10 +62,12 @@ def set_cq_code(code):
     cqCode = "[CQ:%s%s]" % (code["type"], data_str)
     return cqCode
 
-
 def get_cq_code(code_str):
+    """
+    转换 cqCode 字符串为字典
+    """
     code_str = code_str.lstrip("[CQ:").rsplit("]")
-    code_list = code_str.split(",")
+    code_list = code_str[0].split(",")
 
     cq_code = {
         "type": code_list[0],
@@ -25,11 +80,26 @@ def get_cq_code(code_str):
         return cq_code
 
     for code_data in code_list[1:]:
-        key, data = code_data.split("=")
-        cq_code["data"][key] = data
+        key_data = code_data.split("=")
+        if len(key_data) != 2:
+            key_data[1] = "=".join(key_data[1:])
+        cq_code["data"][key_data[0]] = key_data[1]
+
+    if cq_code["type"] == "json":
+        cq_code["data"]["data"] = cqJsonStrToDict(cq_code["data"]["data"])
     
     return cq_code
 
+def cqJsonStrToDict(cq_json_str):
+    """
+    转换 cqCode 中的 json 字符串为字典
+    """
+    cq_json_str = cq_json_str.replace("&#44;", ",")
+    cq_json_str = cq_json_str.replace("&amp;", "&")
+    cq_json_str = cq_json_str.replace("&#91;", "[")
+    cq_json_str = cq_json_str.replace("&#93;", "]")
+
+    return json.loads(cq_json_str)
 
 def face(face_id):
     """
